@@ -21,8 +21,8 @@
 #include <string.h>
 
 #include "gva-error.h"
+#include "gva-mame.h"
 #include "gva-util.h"
-#include "gva-xmame.h"
 
 #define ASSERT_OK(code) \
         if ((code) != SQLITE_OK) g_error ("%s", sqlite3_errmsg (db));
@@ -31,44 +31,61 @@
 #define MAX_ELEMENT_DEPTH 4
 
 #define SQL_CREATE_TABLE_MAME \
-        "CREATE TABLE IF NOT EXISTS mame (build);"
+        "CREATE TABLE IF NOT EXISTS mame (" \
+                "build, " \
+                "debug DEFAULT 'no' " \
+                "CHECK (debug in ('yes', 'no')));"
 
 #define SQL_CREATE_TABLE_GAME \
         "CREATE TABLE IF NOT EXISTS game (" \
                 "name PRIMARY KEY, " \
                 "sourcefile, " \
-                "runnable DEFAULT 'yes', " \
+                "runnable DEFAULT 'yes' " \
+                "CHECK (runnable in ('yes', 'no')), " \
                 "cloneof, " \
                 "romof, " \
-                "romset, " \
+                "romset " \
+                "CHECK (romset in " \
+                "('good', 'best available', 'bad')), " \
                 "sampleof, " \
-                "sampleset, " \
+                "sampleset " \
+                "CHECK (sampleset in " \
+                "('good', 'best available', 'bad')), " \
                 "description NOT NULL, " \
                 "year, " \
                 "manufacturer NOT NULL, " \
-                "history, " \
-                "video_screen NOT NULL, " \
-                "video_orientation, " \
-                "video_width, " \
-                "video_height, " \
-                "video_aspectx, " \
-                "video_aspecty, " \
-                "video_refresh, " \
                 "sound_channels, " \
-                "input_service DEFAULT 'no', " \
-                "input_tilt DEFAULT 'no', " \
+                "input_service DEFAULT 'no' " \
+                "CHECK (input_service in ('yes', 'no')), " \
+                "input_tilt DEFAULT 'no' " \
+                "CHECK (input_tilt in ('yes', 'no')), " \
                 "input_players, " \
-                "input_control, " \
                 "input_buttons, " \
                 "input_coins, " \
-                "driver_status, " \
-                "driver_emulation, " \
-                "driver_color, " \
-                "driver_sound, " \
-                "driver_graphic, " \
-                "driver_cocktail, " \
-                "driver_protection, " \
-                "driver_savestate, " \
+                "driver_status " \
+                "CHECK (driver_status in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_emulation " \
+                "CHECK (driver_emulation in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_color " \
+                "CHECK (driver_color in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_sound " \
+                "CHECK (driver_sound in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_graphic " \
+                "CHECK (driver_graphic in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_cocktail " \
+                "CHECK (driver_cocktail in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_protection " \
+                "CHECK (driver_protection in " \
+                "('good', 'imperfect', 'preliminary')), " \
+                "driver_savestate " \
+                "CHECK (driver_savestate in " \
+                "('supported', 'unsupported')), " \
                 "driver_palettesize);"
 
 #define SQL_INSERT_GAME \
@@ -84,19 +101,10 @@
                 "@description, " \
                 "@year, " \
                 "@manufacturer, " \
-                "@history, " \
-                "@video_screen, " \
-                "@video_orientation, " \
-                "@video_width, " \
-                "@video_height, " \
-                "@video_aspectx, " \
-                "@video_aspecty, " \
-                "@video_refresh, " \
                 "@sound_channels, " \
                 "@input_service, " \
                 "@input_tilt, " \
                 "@input_players, " \
-                "@input_control, " \
                 "@input_buttons, " \
                 "@input_coins, " \
                 "@driver_status, " \
@@ -122,8 +130,8 @@ struct _ParserData
 
         GHashTable *romsets;
         GHashTable *samplesets;
-        GvaProcess *verify_romsets;
-        GvaProcess *verify_samplesets;
+        GvaProcess *verify_roms;
+        GvaProcess *verify_samples;
 
         const gchar *element_stack[MAX_ELEMENT_DEPTH];
         guint element_stack_depth;
@@ -149,7 +157,6 @@ static struct
         const gchar *game;
         const gchar *graphic;
         const gchar *height;
-        const gchar *history;
         const gchar *input;
         const gchar *mame;
         const gchar *manufacturer;
@@ -169,7 +176,6 @@ static struct
         const gchar *sourcefile;
         const gchar *status;
         const gchar *tilt;
-        const gchar *video;
         const gchar *width;
         const gchar *year;
 
@@ -178,7 +184,7 @@ static struct
 static sqlite3 *db = NULL;
 
 static void
-db_parser_bind_text (ParserData *data,
+db_parser_bind_text (sqlite3_stmt *stmt,
                      const gchar *param,
                      const gchar *value)
 {
@@ -186,7 +192,7 @@ db_parser_bind_text (ParserData *data,
         GError *error = NULL;
 
         errcode = sqlite3_bind_text (
-                data->stmt, sqlite3_bind_parameter_index (data->stmt, param),
+                stmt, sqlite3_bind_parameter_index (stmt, param),
                 g_locale_to_utf8 (value, -1, NULL, NULL, &error), -1, g_free);
 
         /* Handle conversion errors. */
@@ -232,7 +238,7 @@ db_parser_start_element_driver (ParserData *data,
                 else
                         continue;
 
-                db_parser_bind_text (data, param, attribute_value[ii]);
+                db_parser_bind_text (data->stmt, param, attribute_value[ii]);
         }
 }
 
@@ -245,7 +251,7 @@ db_parser_start_element_game (ParserData *data,
         gint ii;
 
         /* Bind default values. */
-        db_parser_bind_text (data, "@runnable", "yes");
+        db_parser_bind_text (data->stmt, "@runnable", "yes");
 
         for (ii = 0; attribute_name[ii] != NULL; ii++)
         {
@@ -266,7 +272,7 @@ db_parser_start_element_game (ParserData *data,
                 else
                         continue;
 
-                db_parser_bind_text (data, param, attribute_value[ii]);
+                db_parser_bind_text (data->stmt, param, attribute_value[ii]);
         }
 }
 
@@ -279,8 +285,8 @@ db_parser_start_element_input (ParserData *data,
         gint ii;
 
         /* Bind default values. */
-        db_parser_bind_text (data, "@input_service", "no");
-        db_parser_bind_text (data, "@input_tilt", "no");
+        db_parser_bind_text (data->stmt, "@input_service", "no");
+        db_parser_bind_text (data->stmt, "@input_tilt", "no");
 
         for (ii = 0; attribute_name[ii] != NULL; ii++)
         {
@@ -292,8 +298,6 @@ db_parser_start_element_input (ParserData *data,
                         param = "@input_tilt";
                 else if (attribute_name[ii] == intern.players)
                         param = "@input_players";
-                else if (attribute_name[ii] == intern.control)
-                        param = "@input_control";
                 else if (attribute_name[ii] == intern.buttons)
                         param = "@input_buttons";
                 else if (attribute_name[ii] == intern.coins)
@@ -301,7 +305,7 @@ db_parser_start_element_input (ParserData *data,
                 else
                         continue;
 
-                db_parser_bind_text (data, param, attribute_value[ii]);
+                db_parser_bind_text (data->stmt, param, attribute_value[ii]);
         }
 }
 
@@ -323,7 +327,7 @@ db_parser_start_element_mame (ParserData *data,
                 char *sql;
 
                 sql = sqlite3_mprintf (
-                        "INSERT INTO mame VALUES (%Q)", build);
+                        "INSERT INTO mame (build) VALUES (%Q)", build);
                 gva_db_execute (sql, error);
                 sqlite3_free (sql);
         }
@@ -346,40 +350,7 @@ db_parser_start_element_sound (ParserData *data,
                 else
                         continue;
 
-                db_parser_bind_text (data, param, attribute_value[ii]);
-        }
-}
-
-static void
-db_parser_start_element_video (ParserData *data,
-                               const gchar **attribute_name,
-                               const gchar **attribute_value,
-                               GError **error)
-{
-        gint ii;
-
-        for (ii = 0; attribute_name[ii] != NULL; ii++)
-        {
-                const gchar *param;
-
-                if (attribute_name[ii] == intern.screen)
-                        param = "@video_screen";
-                else if (attribute_name[ii] == intern.orientation)
-                        param = "@video_orientation";
-                else if (attribute_name[ii] == intern.width)
-                        param = "@video_width";
-                else if (attribute_name[ii] == intern.height)
-                        param = "@video_height";
-                else if (attribute_name[ii] == intern.aspectx)
-                        param = "@video_aspectx";
-                else if (attribute_name[ii] == intern.aspecty)
-                        param = "@video_aspecty";
-                else if (attribute_name[ii] == intern.refresh)
-                        param = "@video_refresh";
-                else
-                        continue;
-
-                db_parser_bind_text (data, param, attribute_value[ii]);
+                db_parser_bind_text (data->stmt, param, attribute_value[ii]);
         }
 }
 
@@ -426,10 +397,6 @@ db_parser_start_element (GMarkupParseContext *context,
         else if (element_name == intern.sound)
                 db_parser_start_element_sound (
                         data, attribute_name, attribute_value, error);
-
-        else if (element_name == intern.video)
-                db_parser_start_element_video (
-                        data, attribute_name, attribute_value, error);
 }
 
 static void
@@ -477,16 +444,13 @@ db_parser_text (GMarkupParseContext *context,
         element_name = data->element_stack[data->element_stack_depth - 1];
 
         if (element_name == intern.description)
-                db_parser_bind_text (data, "@description", text);
-
-        else if (element_name == intern.history)
-                db_parser_bind_text (data, "@history", text);
+                db_parser_bind_text (data->stmt, "@description", text);
 
         else if (element_name == intern.manufacturer)
-                db_parser_bind_text (data, "@manufacturer", text);
+                db_parser_bind_text (data->stmt, "@manufacturer", text);
 
         else if (element_name == intern.year)
-                db_parser_bind_text (data, "@year", text);
+                db_parser_bind_text (data->stmt, "@year", text);
 }
 
 static void
@@ -578,13 +542,13 @@ db_parser_data_new (GvaProcess *process)
                 (GDestroyNotify) g_free,
                 (GDestroyNotify) db_verify_string_free);
 
-        data->verify_romsets = gva_xmame_verify_romsets (
-                (GvaXmameCallback) db_verify_insert_status,
+        data->verify_roms = gva_mame_verify_roms (
+                (GvaMameCallback) db_verify_insert_status,
                 data->romsets, &error);
         gva_error_handle (&error);
 
-        data->verify_samplesets = gva_xmame_verify_samplesets (
-                (GvaXmameCallback) db_verify_insert_status,
+        data->verify_samples = gva_mame_verify_samples (
+                (GvaMameCallback) db_verify_insert_status,
                 data->samplesets, &error);
         gva_error_handle (&error);
 
@@ -604,11 +568,11 @@ db_parser_data_free (ParserData *data)
         g_hash_table_destroy (data->romsets);
         g_hash_table_destroy (data->samplesets);
 
-        if (data->verify_romsets != NULL)
-                g_object_unref (data->verify_romsets);
+        if (data->verify_roms != NULL)
+                g_object_unref (data->verify_roms);
 
-        if (data->verify_samplesets != NULL)
-                g_object_unref (data->verify_samplesets);
+        if (data->verify_samples != NULL)
+                g_object_unref (data->verify_samples);
 
         g_slice_free (ParserData, data);
 }
@@ -619,11 +583,16 @@ db_parser_read (GvaProcess *process,
 {
         gchar *line;
 
+        if (process->error != NULL)
+        {
+                gva_process_kill (process);
+                return;
+        }
+
         line = gva_process_stdout_read_line (process);
 
-        if (process->error == NULL)
-                g_markup_parse_context_parse (
-                        data->context, line, -1, &process->error);
+        g_markup_parse_context_parse (
+                data->context, line, -1, &process->error);
 
         g_free (line);
 }
@@ -633,23 +602,35 @@ db_parser_exit (GvaProcess *process,
                 gint status,
                 ParserData *data)
 {
-        GTimeVal time_elapsed;
+        GError *error = NULL;
 
         if (process->error == NULL)
+        {
+                GTimeVal time_elapsed;
+
                 g_markup_parse_context_end_parse (
                         data->context, &process->error);
 
-        db_verify_update_status (
-                data->verify_romsets, data->romsets, "romset");
-        db_verify_update_status (
-                data->verify_samplesets, data->samplesets, "sampleset");
-        gva_db_execute (SQL_DELETE_NOT_FOUND, &process->error);
+                db_verify_update_status (
+                        data->verify_roms, data->romsets, "romset");
+                db_verify_update_status (
+                        data->verify_samples, data->samplesets, "sampleset");
+                gva_db_execute (SQL_DELETE_NOT_FOUND, &process->error);
 
-        gva_process_get_time_elapsed (process, &time_elapsed);
+                gva_db_transaction_commit (&error);
+                gva_error_handle (&error);
 
-        g_message (
-                "Database built in %d.%d seconds.",
-                time_elapsed.tv_sec, time_elapsed.tv_usec / 100000);
+                gva_process_get_time_elapsed (process, &time_elapsed);
+
+                g_message (
+                        "Database built in %d.%d seconds.",
+                        time_elapsed.tv_sec, time_elapsed.tv_usec / 100000);
+        }
+        else
+        {
+                gva_db_transaction_rollback (&error);
+                gva_error_handle (&error);
+        }
 
         db_parser_data_free (data);
 }
@@ -732,7 +713,6 @@ gva_db_build (GError **error)
         intern.game         = g_intern_static_string ("game");
         intern.graphic      = g_intern_static_string ("graphic");
         intern.height       = g_intern_static_string ("height");
-        intern.history      = g_intern_static_string ("history");
         intern.input        = g_intern_static_string ("input");
         intern.mame         = g_intern_static_string ("mame");
         intern.manufacturer = g_intern_static_string ("manufacturer");
@@ -752,17 +732,22 @@ gva_db_build (GError **error)
         intern.sourcefile   = g_intern_static_string ("sourcefile");
         intern.status       = g_intern_static_string ("status");
         intern.tilt         = g_intern_static_string ("tilt");
-        intern.video        = g_intern_static_string ("video");
         intern.width        = g_intern_static_string ("width");
         intern.year         = g_intern_static_string ("year");
 
         if (!gva_db_reset (error))
                 return NULL;
 
-        process = gva_xmame_list_xml (error);
+        process = gva_mame_list_xml (error);
         if (process == NULL)
                 return NULL;
 
+        if (!gva_db_transaction_begin (error))
+        {
+                g_object_unref (process);
+                return NULL;
+        }
+
         data = db_parser_data_new (process);
 
         g_signal_connect (
@@ -815,6 +800,24 @@ gva_db_execute (const gchar *sql,
         return (errcode == SQLITE_OK);
 }
 
+gboolean
+gva_db_transaction_begin (GError **error)
+{
+        return gva_db_execute ("BEGIN TRANSACTION", error);
+}
+
+gboolean
+gva_db_transaction_commit (GError **error)
+{
+        return gva_db_execute ("COMMIT TRANSACTION", error);
+}
+
+gboolean
+gva_db_transaction_rollback (GError **error)
+{
+        return gva_db_execute ("ROLLBACK TRANSACTION", error);
+}
+
 gboolean
 gva_db_prepare (const gchar *sql,
                 sqlite3_stmt **stmt,
@@ -881,7 +884,7 @@ gva_db_get_filename (void)
 
         if (G_UNLIKELY (filename == NULL))
                 filename = g_build_filename (
-                        g_get_user_data_dir (), PACKAGE ".db", NULL);
+                        gva_get_user_data_dir (), "games.db", NULL);
 
         return filename;
 }
@@ -915,7 +918,7 @@ gva_db_needs_rebuilt (void)
         TEST_CASE (db_build_id == NULL);
 
         reason = "the MAME version could not be determined";
-        mame_version = gva_xmame_get_version (&error);
+        mame_version = gva_mame_get_version (&error);
         gva_error_handle (&error);
         TEST_CASE (mame_version == NULL);
 
